(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[892],{7476:function(e,n,r){(window.__NEXT_P=window.__NEXT_P||[]).push(["/users",function(){return r(85126)}])},85126:function(e,n,r){"use strict";r.r(n),r.d(n,{default:function(){return o}});var t=r(85893);r(67294);var s=r(9008),u=r.n(s),i=r(5152);let c=r.n(i)()(()=>Promise.all([r.e(739),r.e(850),r.e(683),r.e(329),r.e(243),r.e(669)]).then(r.bind(r,27669)).then(e=>e.Users),{loadableGenerated:{webpack:()=>[27669]},ssr:!1});function o(){return(0,t.jsxs)(t.Fragment,{children:[(0,t.jsx)(u(),{children:(0,t.jsx)("title",{children:"Users | SkyPilot Dashboard"})}),(0,t.jsx)(c,{})]})}},9008:function(e,n,r){e.exports=r(37219)}},function(e){e.O(0,[888,774,179],function(){return e(e.s=7476)}),_N_E=e.O()}]);