self.__BUILD_MANIFEST=function(s,c,e,a,t,b,u){return{__rewrites:{afterFiles:[],beforeFiles:[],fallback:[]},"/":["static/chunks/pages/index-bbfee9b059a805b6.js"],"/_error":["static/chunks/pages/_error-6ae619510b1539d6.js"],"/clusters":["static/chunks/pages/clusters-d29537c69bc99522.js"],"/clusters/[cluster]":[s,e,a,c,t,b,u,"static/chunks/37-4be0ca7b96902ef8.js","static/chunks/pages/clusters/[cluster]-5b1e4346e43739c0.js"],"/clusters/[cluster]/[job]":[s,c,"static/chunks/pages/clusters/[cluster]/[job]-063a4b1ca0943b56.js"],"/config":["static/chunks/pages/config-2558eca3f36945b5.js"],"/infra":["static/chunks/pages/infra-adeedccddb7487ff.js"],"/infra/[context]":["static/chunks/pages/infra/[context]-1c94b03dbe9be2b6.js"],"/jobs":["static/chunks/pages/jobs-4c89bbdc9396dbe8.js"],"/jobs/pools/[pool]":[s,e,a,"static/chunks/256-659f45566d614855.js",c,t,"static/chunks/pages/jobs/pools/[pool]-d1177c6d83abc273.js"],"/jobs/[job]":[s,e,a,c,"static/chunks/pages/jobs/[job]-d92cbe3badf8118e.js"],"/jobs/[job]/[task]":[s,c,"static/chunks/pages/jobs/[job]/[task]-7a11a44284d173ba.js"],"/plugins/[...slug]":[s,"static/chunks/pages/plugins/[...slug]-79220cd0c082e6a8.js"],"/recipes":["static/chunks/pages/recipes-6512773e224e7802.js"],"/recipes/[recipe]":["static/chunks/pages/recipes/[recipe]-7a11abdeebbd10aa.js"],"/users":["static/chunks/pages/users-9c3008f1f5bfc99d.js"],"/volumes":["static/chunks/pages/volumes-2ee2a641c7451957.js"],"/workspace/new":["static/chunks/pages/workspace/new-b91248b19db73aed.js"],"/workspaces":["static/chunks/pages/workspaces-df5e2e71aeb337bf.js"],"/workspaces/[name]":[s,e,c,t,b,u,"static/chunks/899-8c32ce645be8f995.js","static/chunks/pages/workspaces/[name]-6cee3ca73ea7a283.js"],"/[...path]":[s,"static/chunks/pages/[...path]-689e2626305d9c7e.js"],sortedPages:["/","/_app","/_error","/clusters","/clusters/[cluster]","/clusters/[cluster]/[job]","/config","/infra","/infra/[context]","/jobs","/jobs/pools/[pool]","/jobs/[job]","/jobs/[job]/[task]","/plugins/[...slug]","/recipes","/recipes/[recipe]","/users","/volumes","/workspace/new","/workspaces","/workspaces/[name]","/[...path]"]}}("static/chunks/739-d30c174396b467eb.js","static/chunks/850-2931ff979121bbd5.js","static/chunks/798-c0525dc3f21e488d.js","static/chunks/417-9ada6598d3096dd9.js","static/chunks/683-9fb45111dbd7c95f.js","static/chunks/329-6230b4558aedc08d.js","static/chunks/243-148167af5a23f461.js"),self.__BUILD_MANIFEST_CB&&self.__BUILD_MANIFEST_CB();