(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[144],{43969:function(n,e,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/volumes",function(){return t(32567)}])},32567:function(n,e,t){"use strict";t.r(e),t.d(e,{default:function(){return l}});var r=t(85893);t(67294);var u=t(9008),s=t.n(u),i=t(5152);let o=t.n(i)()(()=>Promise.all([t.e(739),t.e(256),t.e(850),t.e(725)]).then(t.bind(t,54725)).then(n=>n.Volumes),{loadableGenerated:{webpack:()=>[54725]},ssr:!1});function l(){return(0,r.jsxs)(r.Fragment,{children:[(0,r.jsx)(s(),{children:(0,r.jsx)("title",{children:"Volumes | SkyPilot Dashboard"})}),(0,r.jsx)(o,{})]})}},9008:function(n,e,t){n.exports=t(37219)}},function(n){n.O(0,[888,774,179],function(){return n(n.s=43969)}),_N_E=n.O()}]);