(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[241],{40458:function(e,n,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/recipes",function(){return t(21091)}])},21091:function(e,n,t){"use strict";t.r(n),t.d(n,{default:function(){return o}});var r=t(85893);t(67294);var i=t(9008),s=t.n(i),u=t(5152);let c=t.n(u)()(()=>Promise.all([t.e(739),t.e(798),t.e(400)]).then(t.bind(t,61400)).then(e=>e.RecipeHub),{loadableGenerated:{webpack:()=>[61400]},ssr:!1});function o(){return(0,r.jsxs)(r.Fragment,{children:[(0,r.jsx)(s(),{children:(0,r.jsx)("title",{children:"Recipes | SkyPilot Dashboard"})}),(0,r.jsx)(c,{})]})}},9008:function(e,n,t){e.exports=t(37219)}},function(e){e.O(0,[888,774,179],function(){return e(e.s=40458)}),_N_E=e.O()}]);