# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sky/schemas/generated/autostopv1.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&sky/schemas/generated/autostopv1.proto\x12\x0b\x61utostop.v1\"\xc1\x01\n\x12SetAutostopRequest\x12\x14\n\x0cidle_minutes\x18\x01 \x01(\x05\x12\x0f\n\x07\x62\x61\x63kend\x18\x02 \x01(\t\x12.\n\x08wait_for\x18\x03 \x01(\x0e\x32\x1c.autostop.v1.AutostopWaitFor\x12\x0c\n\x04\x64own\x18\x04 \x01(\x08\x12\x11\n\x04hook\x18\x05 \x01(\tH\x00\x88\x01\x01\x12\x19\n\x0chook_timeout\x18\x06 \x01(\x05H\x01\x88\x01\x01\x42\x07\n\x05_hookB\x0f\n\r_hook_timeout\"\x15\n\x13SetAutostopResponse\"\x17\n\x15IsAutostoppingRequest\"1\n\x16IsAutostoppingResponse\x12\x17\n\x0fis_autostopping\x18\x01 \x01(\x08*\x90\x01\n\x0f\x41utostopWaitFor\x12!\n\x1d\x41UTOSTOP_WAIT_FOR_UNSPECIFIED\x10\x00\x12\"\n\x1e\x41UTOSTOP_WAIT_FOR_JOBS_AND_SSH\x10\x01\x12\x1a\n\x16\x41UTOSTOP_WAIT_FOR_JOBS\x10\x02\x12\x1a\n\x16\x41UTOSTOP_WAIT_FOR_NONE\x10\x03\x32\xbe\x01\n\x0f\x41utostopService\x12P\n\x0bSetAutostop\x12\x1f.autostop.v1.SetAutostopRequest\x1a .autostop.v1.SetAutostopResponse\x12Y\n\x0eIsAutostopping\x12\".autostop.v1.IsAutostoppingRequest\x1a#.autostop.v1.IsAutostoppingResponseb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'sky.schemas.generated.autostopv1_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_AUTOSTOPWAITFOR']._serialized_start=351
  _globals['_AUTOSTOPWAITFOR']._serialized_end=495
  _globals['_SETAUTOSTOPREQUEST']._serialized_start=56
  _globals['_SETAUTOSTOPREQUEST']._serialized_end=249
  _globals['_SETAUTOSTOPRESPONSE']._serialized_start=251
  _globals['_SETAUTOSTOPRESPONSE']._serialized_end=272
  _globals['_ISAUTOSTOPPINGREQUEST']._serialized_start=274
  _globals['_ISAUTOSTOPPINGREQUEST']._serialized_end=297
  _globals['_ISAUTOSTOPPINGRESPONSE']._serialized_start=299
  _globals['_ISAUTOSTOPPINGRESPONSE']._serialized_end=348
  _globals['_AUTOSTOPSERVICE']._serialized_start=498
  _globals['_AUTOSTOPSERVICE']._serialized_end=688
# @@protoc_insertion_point(module_scope)
