(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[310],{74118:function(e,n,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/recipes/[recipe]",function(){return t(53399)}])},53399:function(e,n,t){"use strict";t.r(n),t.d(n,{default:function(){return l}});var r=t(85893);t(67294);var i=t(9008),c=t.n(i),s=t(5152),u=t.n(s),o=t(11163);let a=u()(()=>Promise.all([t.e(739),t.e(798),t.e(417),t.e(940)]).then(t.bind(t,50940)).then(e=>e.RecipeDetail),{loadableGenerated:{webpack:()=>[50940]},ssr:!1});function l(){let{recipe:e}=(0,o.useRouter)().query;return(0,r.jsxs)(r.Fragment,{children:[(0,r.jsx)(c(),{children:(0,r.jsx)("title",{children:e?"".concat(e," | Recipes | SkyPilot Dashboard"):"Recipes | SkyPilot Dashboard"})}),(0,r.jsx)(a,{})]})}},9008:function(e,n,t){e.exports=t(37219)}},function(e){e.O(0,[888,774,179],function(){return e(e.s=74118)}),_N_E=e.O()}]);