(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[324],{77614:function(n,e,u){(window.__NEXT_P=window.__NEXT_P||[]).push(["/workspaces/[name]",function(){return u(59431)}])},59431:function(n,e,u){"use strict";u.r(e),u.d(e,{default:function(){return i}});var r=u(85893);u(67294);var t=u(11163),o=u(63195);function i(){let n=(0,t.useRouter)(),{name:e}=n.query;return n.isReady&&e?(0,r.jsx)(o.WorkspaceEditor,{workspaceName:e}):(0,r.jsx)("div",{children:"Loading..."})}},9008:function(n,e,u){n.exports=u(37219)}},function(n){n.O(0,[739,798,850,683,329,243,899,888,774,179],function(){return n(n.s=77614)}),_N_E=n.O()}]);