(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[963],{35113:function(n,e,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/workspace/new",function(){return t(4250)}])},4250:function(n,e,t){"use strict";t.r(e),t.d(e,{default:function(){return o}});var r=t(85893);t(67294);var s=t(9008),c=t.n(s),u=t(5152);let i=t.n(u)()(()=>t.e(785).then(t.bind(t,33785)).then(n=>n.NewWorkspace),{loadableGenerated:{webpack:()=>[33785]},ssr:!1});function o(){return(0,r.jsxs)(r.Fragment,{children:[(0,r.jsx)(c(),{children:(0,r.jsx)("title",{children:"New Workspace | SkyPilot Dashboard"})}),(0,r.jsx)(i,{})]})}},9008:function(n,e,t){n.exports=t(37219)}},function(n){n.O(0,[888,774,179],function(){return n(n.s=35113)}),_N_E=n.O()}]);