(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[588],{56206:function(n,e,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/infra",function(){return t(89054)}])},89054:function(n,e,t){"use strict";t.r(e),t.d(e,{default:function(){return c}});var r=t(85893);t(67294);var i=t(9008),s=t.n(i),u=t(5152);let a=t.n(u)()(()=>Promise.all([t.e(739),t.e(850),t.e(329),t.e(495)]).then(t.bind(t,23495)).then(n=>n.GPUs),{loadableGenerated:{webpack:()=>[23495]},ssr:!1});function c(){return(0,r.jsxs)(r.Fragment,{children:[(0,r.jsx)(s(),{children:(0,r.jsx)("title",{children:"Infra | SkyPilot Dashboard"})}),(0,r.jsx)(a,{})]})}},9008:function(n,e,t){n.exports=t(37219)}},function(n){n.O(0,[888,774,179],function(){return n(n.s=56206)}),_N_E=n.O()}]);