# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sky/schemas/generated/managed_jobsv1.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*sky/schemas/generated/managed_jobsv1.proto\x12\x0fmanaged_jobs.v1\"\x15\n\x06JobIds\x12\x0b\n\x03ids\x18\x01 \x03(\x03\"\x1c\n\nUserHashes\x12\x0e\n\x06hashes\x18\x01 \x03(\t\"\x1c\n\x08Statuses\x12\x10\n\x08statuses\x18\x01 \x03(\t\"\x18\n\x06\x46ields\x12\x0e\n\x06\x66ields\x18\x01 \x03(\t\" \n\nWorkspaces\x12\x12\n\nworkspaces\x18\x01 \x03(\t\"\x13\n\x11GetVersionRequest\"0\n\x12GetVersionResponse\x12\x1a\n\x12\x63ontroller_version\x18\x01 \x01(\t\"\xab\x05\n\x12GetJobTableRequest\x12\x15\n\rskip_finished\x18\x01 \x01(\x08\x12?\n\x15\x61\x63\x63\x65ssible_workspaces\x18\x02 \x01(\x0b\x32\x1b.managed_jobs.v1.WorkspacesH\x00\x88\x01\x01\x12-\n\x07job_ids\x18\x03 \x01(\x0b\x32\x17.managed_jobs.v1.JobIdsH\x01\x88\x01\x01\x12\x1c\n\x0fworkspace_match\x18\x04 \x01(\tH\x02\x88\x01\x01\x12\x17\n\nname_match\x18\x05 \x01(\tH\x03\x88\x01\x01\x12\x17\n\npool_match\x18\x06 \x01(\tH\x04\x88\x01\x01\x12\x11\n\x04page\x18\x07 \x01(\x05H\x05\x88\x01\x01\x12\x12\n\x05limit\x18\x08 \x01(\x05H\x06\x88\x01\x01\x12\x35\n\x0buser_hashes\x18\t \x01(\x0b\x32\x1b.managed_jobs.v1.UserHashesH\x07\x88\x01\x01\x12\x30\n\x08statuses\x18\n \x01(\x0b\x32\x19.managed_jobs.v1.StatusesH\x08\x88\x01\x01\x12#\n\x1bshow_jobs_without_user_hash\x18\x0b \x01(\x08\x12,\n\x06\x66ields\x18\x0c \x01(\x0b\x32\x17.managed_jobs.v1.FieldsH\t\x88\x01\x01\x12\x14\n\x07sort_by\x18\r \x01(\tH\n\x88\x01\x01\x12\x17\n\nsort_order\x18\x0e \x01(\tH\x0b\x88\x01\x01\x42\x18\n\x16_accessible_workspacesB\n\n\x08_job_idsB\x12\n\x10_workspace_matchB\r\n\x0b_name_matchB\r\n\x0b_pool_matchB\x07\n\x05_pageB\x08\n\x06_limitB\x0e\n\x0c_user_hashesB\x0b\n\t_statusesB\t\n\x07_fieldsB\n\n\x08_sort_byB\r\n\x0b_sort_order\"\xbc\x0b\n\x0eManagedJobInfo\x12\x0e\n\x06job_id\x18\x01 \x01(\x03\x12\x0f\n\x07task_id\x18\x02 \x01(\x03\x12\x10\n\x08job_name\x18\x03 \x01(\t\x12\x11\n\ttask_name\x18\x04 \x01(\t\x12\x14\n\x0cjob_duration\x18\x05 \x01(\x01\x12\x16\n\tworkspace\x18\x06 \x01(\tH\x00\x88\x01\x01\x12\x31\n\x06status\x18\x07 \x01(\x0e\x32!.managed_jobs.v1.ManagedJobStatus\x12@\n\x0eschedule_state\x18\x08 \x01(\x0e\x32(.managed_jobs.v1.ManagedJobScheduleState\x12\x11\n\tresources\x18\t \x01(\t\x12\x19\n\x11\x63luster_resources\x18\n \x01(\t\x12\x1e\n\x16\x63luster_resources_full\x18\x0b \x01(\t\x12\r\n\x05\x63loud\x18\x0c \x01(\t\x12\x0e\n\x06region\x18\r \x01(\t\x12\r\n\x05infra\x18\x0e \x01(\t\x12G\n\x0c\x61\x63\x63\x65lerators\x18\x0f \x03(\x0b\x32\x31.managed_jobs.v1.ManagedJobInfo.AcceleratorsEntry\x12\x16\n\x0erecovery_count\x18\x10 \x01(\x05\x12\x14\n\x07\x64\x65tails\x18\x11 \x01(\tH\x01\x88\x01\x01\x12\x1b\n\x0e\x66\x61ilure_reason\x18\x12 \x01(\tH\x02\x88\x01\x01\x12\x16\n\tuser_name\x18\x13 \x01(\tH\x03\x88\x01\x01\x12\x16\n\tuser_hash\x18\x14 \x01(\tH\x04\x88\x01\x01\x12\x19\n\x0csubmitted_at\x18\x15 \x01(\x01H\x05\x88\x01\x01\x12\x15\n\x08start_at\x18\x16 \x01(\x01H\x06\x88\x01\x01\x12\x13\n\x06\x65nd_at\x18\x17 \x01(\x01H\x07\x88\x01\x01\x12\x16\n\tuser_yaml\x18\x18 \x01(\tH\x08\x88\x01\x01\x12\x17\n\nentrypoint\x18\x19 \x01(\tH\t\x88\x01\x01\x12?\n\x08metadata\x18\x1a \x03(\x0b\x32-.managed_jobs.v1.ManagedJobInfo.MetadataEntry\x12\x11\n\x04pool\x18\x1b \x01(\tH\n\x88\x01\x01\x12\x16\n\tpool_hash\x18\x1c \x01(\tH\x0b\x88\x01\x01\x12\x14\n\x07_job_id\x18\x1d \x01(\x03H\x0c\x88\x01\x01\x12\x39\n\x05links\x18\x1e \x03(\x0b\x32*.managed_jobs.v1.ManagedJobInfo.LinksEntry\x12$\n\x17is_primary_in_job_group\x18\x1f \x01(\x08H\r\x88\x01\x01\x12\x11\n\x04zone\x18  \x01(\tH\x0e\x88\x01\x01\x12;\n\x06labels\x18! \x03(\x0b\x32+.managed_jobs.v1.ManagedJobInfo.LabelsEntry\x12\"\n\x15\x63luster_name_on_cloud\x18\" \x01(\tH\x0f\x88\x01\x01\x1a\x33\n\x11\x41\x63\x63\x65leratorsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a,\n\nLinksEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x0c\n\n_workspaceB\n\n\x08_detailsB\x11\n\x0f_failure_reasonB\x0c\n\n_user_nameB\x0c\n\n_user_hashB\x0f\n\r_submitted_atB\x0b\n\t_start_atB\t\n\x07_end_atB\x0c\n\n_user_yamlB\r\n\x0b_entrypointB\x07\n\x05_poolB\x0c\n\n_pool_hashB\n\n\x08X_job_idB\x1a\n\x18_is_primary_in_job_groupB\x07\n\x05_zoneB\x18\n\x16_cluster_name_on_cloud\"\xf0\x01\n\x13GetJobTableResponse\x12-\n\x04jobs\x18\x01 \x03(\x0b\x32\x1f.managed_jobs.v1.ManagedJobInfo\x12\r\n\x05total\x18\x02 \x01(\x05\x12\x17\n\x0ftotal_no_filter\x18\x03 \x01(\x05\x12M\n\rstatus_counts\x18\x04 \x03(\x0b\x32\x36.managed_jobs.v1.GetJobTableResponse.StatusCountsEntry\x1a\x33\n\x11StatusCountsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\"?\n\x19GetAllJobIdsByNameRequest\x12\x15\n\x08job_name\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x0b\n\t_job_name\"-\n\x1aGetAllJobIdsByNameResponse\x12\x0f\n\x07job_ids\x18\x01 \x03(\x03\"\xaf\x02\n\x11\x43\x61ncelJobsRequest\x12\x19\n\x11\x63urrent_workspace\x18\x01 \x01(\t\x12\x16\n\tuser_hash\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x13\n\tall_users\x18\x03 \x01(\x08H\x00\x12*\n\x07job_ids\x18\x04 \x01(\x0b\x32\x17.managed_jobs.v1.JobIdsH\x00\x12\x12\n\x08job_name\x18\x05 \x01(\tH\x00\x12\x13\n\tpool_name\x18\x06 \x01(\tH\x00\x12\x15\n\x08graceful\x18\x07 \x01(\x08H\x02\x88\x01\x01\x12\x1d\n\x10graceful_timeout\x18\x08 \x01(\x03H\x03\x88\x01\x01\x42\x17\n\x15\x63\x61ncellation_criteriaB\x0c\n\n_user_hashB\x0b\n\t_gracefulB\x13\n\x11_graceful_timeout\"%\n\x12\x43\x61ncelJobsResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\"\x97\x01\n\x11StreamLogsRequest\x12\x15\n\x08job_name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x06job_id\x18\x02 \x01(\x03H\x01\x88\x01\x01\x12\x0e\n\x06\x66ollow\x18\x03 \x01(\x08\x12\x12\n\ncontroller\x18\x04 \x01(\x08\x12\x11\n\x04tail\x18\x05 \x01(\x05H\x02\x88\x01\x01\x42\x0b\n\t_job_nameB\t\n\x07_job_idB\x07\n\x05_tail\"L\n\x12StreamLogsResponse\x12\x10\n\x08log_line\x18\x01 \x01(\t\x12\x16\n\texit_code\x18\x02 \x01(\x05H\x00\x88\x01\x01\x42\x0c\n\n_exit_code*\x85\x04\n\x10ManagedJobStatus\x12\"\n\x1eMANAGED_JOB_STATUS_UNSPECIFIED\x10\x00\x12\x1e\n\x1aMANAGED_JOB_STATUS_PENDING\x10\x01\x12 \n\x1cMANAGED_JOB_STATUS_SUBMITTED\x10\x02\x12\x1f\n\x1bMANAGED_JOB_STATUS_STARTING\x10\x03\x12\x1e\n\x1aMANAGED_JOB_STATUS_RUNNING\x10\x04\x12!\n\x1dMANAGED_JOB_STATUS_RECOVERING\x10\x05\x12!\n\x1dMANAGED_JOB_STATUS_CANCELLING\x10\x06\x12 \n\x1cMANAGED_JOB_STATUS_SUCCEEDED\x10\x07\x12 \n\x1cMANAGED_JOB_STATUS_CANCELLED\x10\x08\x12\x1d\n\x19MANAGED_JOB_STATUS_FAILED\x10\t\x12#\n\x1fMANAGED_JOB_STATUS_FAILED_SETUP\x10\n\x12\'\n#MANAGED_JOB_STATUS_FAILED_PRECHECKS\x10\x0b\x12)\n%MANAGED_JOB_STATUS_FAILED_NO_RESOURCE\x10\x0c\x12(\n$MANAGED_JOB_STATUS_FAILED_CONTROLLER\x10\r*\x9e\x03\n\x17ManagedJobScheduleState\x12*\n&MANAGED_JOB_SCHEDULE_STATE_UNSPECIFIED\x10\x00\x12\x35\n-DEPRECATED_MANAGED_JOB_SCHEDULE_STATE_INVALID\x10\x01\x1a\x02\x08\x01\x12\'\n#MANAGED_JOB_SCHEDULE_STATE_INACTIVE\x10\x02\x12&\n\"MANAGED_JOB_SCHEDULE_STATE_WAITING\x10\x03\x12,\n(MANAGED_JOB_SCHEDULE_STATE_ALIVE_WAITING\x10\x04\x12(\n$MANAGED_JOB_SCHEDULE_STATE_LAUNCHING\x10\x05\x12,\n(MANAGED_JOB_SCHEDULE_STATE_ALIVE_BACKOFF\x10\x06\x12$\n MANAGED_JOB_SCHEDULE_STATE_ALIVE\x10\x07\x12#\n\x1fMANAGED_JOB_SCHEDULE_STATE_DONE\x10\x08\x32\xe4\x03\n\x12ManagedJobsService\x12U\n\nGetVersion\x12\".managed_jobs.v1.GetVersionRequest\x1a#.managed_jobs.v1.GetVersionResponse\x12X\n\x0bGetJobTable\x12#.managed_jobs.v1.GetJobTableRequest\x1a$.managed_jobs.v1.GetJobTableResponse\x12m\n\x12GetAllJobIdsByName\x12*.managed_jobs.v1.GetAllJobIdsByNameRequest\x1a+.managed_jobs.v1.GetAllJobIdsByNameResponse\x12U\n\nCancelJobs\x12\".managed_jobs.v1.CancelJobsRequest\x1a#.managed_jobs.v1.CancelJobsResponse\x12W\n\nStreamLogs\x12\".managed_jobs.v1.StreamLogsRequest\x1a#.managed_jobs.v1.StreamLogsResponse0\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'sky.schemas.generated.managed_jobsv1_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_MANAGEDJOBSCHEDULESTATE'].values_by_name["DEPRECATED_MANAGED_JOB_SCHEDULE_STATE_INVALID"]._loaded_options = None
  _globals['_MANAGEDJOBSCHEDULESTATE'].values_by_name["DEPRECATED_MANAGED_JOB_SCHEDULE_STATE_INVALID"]._serialized_options = b'\010\001'
  _globals['_MANAGEDJOBINFO_ACCELERATORSENTRY']._loaded_options = None
  _globals['_MANAGEDJOBINFO_ACCELERATORSENTRY']._serialized_options = b'8\001'
  _globals['_MANAGEDJOBINFO_METADATAENTRY']._loaded_options = None
  _globals['_MANAGEDJOBINFO_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_MANAGEDJOBINFO_LINKSENTRY']._loaded_options = None
  _globals['_MANAGEDJOBINFO_LINKSENTRY']._serialized_options = b'8\001'
  _globals['_MANAGEDJOBINFO_LABELSENTRY']._loaded_options = None
  _globals['_MANAGEDJOBINFO_LABELSENTRY']._serialized_options = b'8\001'
  _globals['_GETJOBTABLERESPONSE_STATUSCOUNTSENTRY']._loaded_options = None
  _globals['_GETJOBTABLERESPONSE_STATUSCOUNTSENTRY']._serialized_options = b'8\001'
  _globals['_MANAGEDJOBSTATUS']._serialized_start=3367
  _globals['_MANAGEDJOBSTATUS']._serialized_end=3884
  _globals['_MANAGEDJOBSCHEDULESTATE']._serialized_start=3887
  _globals['_MANAGEDJOBSCHEDULESTATE']._serialized_end=4301
  _globals['_JOBIDS']._serialized_start=63
  _globals['_JOBIDS']._serialized_end=84
  _globals['_USERHASHES']._serialized_start=86
  _globals['_USERHASHES']._serialized_end=114
  _globals['_STATUSES']._serialized_start=116
  _globals['_STATUSES']._serialized_end=144
  _globals['_FIELDS']._serialized_start=146
  _globals['_FIELDS']._serialized_end=170
  _globals['_WORKSPACES']._serialized_start=172
  _globals['_WORKSPACES']._serialized_end=204
  _globals['_GETVERSIONREQUEST']._serialized_start=206
  _globals['_GETVERSIONREQUEST']._serialized_end=225
  _globals['_GETVERSIONRESPONSE']._serialized_start=227
  _globals['_GETVERSIONRESPONSE']._serialized_end=275
  _globals['_GETJOBTABLEREQUEST']._serialized_start=278
  _globals['_GETJOBTABLEREQUEST']._serialized_end=961
  _globals['_MANAGEDJOBINFO']._serialized_start=964
  _globals['_MANAGEDJOBINFO']._serialized_end=2432
  _globals['_MANAGEDJOBINFO_ACCELERATORSENTRY']._serialized_start=1998
  _globals['_MANAGEDJOBINFO_ACCELERATORSENTRY']._serialized_end=2049
  _globals['_MANAGEDJOBINFO_METADATAENTRY']._serialized_start=2051
  _globals['_MANAGEDJOBINFO_METADATAENTRY']._serialized_end=2098
  _globals['_MANAGEDJOBINFO_LINKSENTRY']._serialized_start=2100
  _globals['_MANAGEDJOBINFO_LINKSENTRY']._serialized_end=2144
  _globals['_MANAGEDJOBINFO_LABELSENTRY']._serialized_start=2146
  _globals['_MANAGEDJOBINFO_LABELSENTRY']._serialized_end=2191
  _globals['_GETJOBTABLERESPONSE']._serialized_start=2435
  _globals['_GETJOBTABLERESPONSE']._serialized_end=2675
  _globals['_GETJOBTABLERESPONSE_STATUSCOUNTSENTRY']._serialized_start=2624
  _globals['_GETJOBTABLERESPONSE_STATUSCOUNTSENTRY']._serialized_end=2675
  _globals['_GETALLJOBIDSBYNAMEREQUEST']._serialized_start=2677
  _globals['_GETALLJOBIDSBYNAMEREQUEST']._serialized_end=2740
  _globals['_GETALLJOBIDSBYNAMERESPONSE']._serialized_start=2742
  _globals['_GETALLJOBIDSBYNAMERESPONSE']._serialized_end=2787
  _globals['_CANCELJOBSREQUEST']._serialized_start=2790
  _globals['_CANCELJOBSREQUEST']._serialized_end=3093
  _globals['_CANCELJOBSRESPONSE']._serialized_start=3095
  _globals['_CANCELJOBSRESPONSE']._serialized_end=3132
  _globals['_STREAMLOGSREQUEST']._serialized_start=3135
  _globals['_STREAMLOGSREQUEST']._serialized_end=3286
  _globals['_STREAMLOGSRESPONSE']._serialized_start=3288
  _globals['_STREAMLOGSRESPONSE']._serialized_end=3364
  _globals['_MANAGEDJOBSSERVICE']._serialized_start=4304
  _globals['_MANAGEDJOBSSERVICE']._serialized_end=4788
# @@protoc_insertion_point(module_scope)
